# template for PSet #3, Python Task #1
import numpy
import matplotlib.pyplot as p
import PS3_tests

# arguments:
#   channel -- instance of the PS3_tests.channel class
#   max_length -- integer
#   tol -- float
# return value:
#   a voltage sequence of length max_length or less
def unit_sample_response(channel,max_length=1000,tol=0.005):
    """
    Returns sequence of samples that corresponds to the unit-sample
    response (USR) of the channel.

    channel is function you call with an input sequence of voltage
    samples.  It returns a sequence of voltage values, which is the
    response of the channel to that input.
    
    max_length sets the length of the test waveform to be sent through
    the channel.

    The voltage sample sequence representing the unit sample response
    should truncated to the smallest length such that the maximum
    magnitude of the truncated samples are smaller than tol times the
    value that is the largest magnitude sample in the unit sample
    response. Please make sure that if tol=0, the return USR should
    have length equal to max_length.
    """
    input = [1] + [0]*(max_length)
    #print input
    output = channel(input)
    
    #print output
    if tol < 0:
        return 'error'
    elif tol > 0:
        h_max = max(output)
        #look from the back of the list
        x = max_length
        for i in range(max_length, 0, -1):
            if output[i] >= tol*h_max:
                return output[0:i+1]
    return output[0:max_length]

if __name__ == '__main__':
    # Create the channels (noise free)
    channel0 = PS3_tests.channel(channelid='0')
    channel1 = PS3_tests.channel(channelid='1')
    channel2 = PS3_tests.channel(channelid='2')

    # plot the unit-sample response of our three virtual channels
    PS3_tests.plot_USR(unit_sample_response(channel0),'0')
    PS3_tests.plot_USR(unit_sample_response(channel1),'1')
    PS3_tests.plot_USR(unit_sample_response(channel2),'2')

    p.show()
