# this is the template for PSet #4, Python Task #2
import matplotlib.pyplot as p
import math,numpy,random
import PS4_tests
import PS4_1

def receive(samples,samples_per_bit=4,vth=0.5):
    """
    Apply a statistical measure to samples to determine which
    sample in the bit cell should be used to determine the
    transmitted message bit.  vth is the digitization threshold.
    Return a sequence or array of received message bits.
    """
    sample_stats = PS4_1.sample_stats(samples,samples_per_bit,vth)
    #print sample_stats
    best_avg = None
    best_avg_index = None
    i = 0
    for bit in sample_stats:
        #print bit
        if best_avg < bit[1]:
            best_avg = bit[1]
            best_avg_index = i
        i=i+1
    
    #print best_avg_index

    bins = numpy.reshape(numpy.array(samples), (-1,samples_per_bit))
    samples = bins[:,best_avg_index]

    string = []

    for i in samples:
        #print i
        if i>vth:
            string.append(1)
        else:
            string.append(0)
    #print vth
    #print string
    return string

def bit_error_rate(seq1,seq2):
    """
    Perform a bit-by-bit comparison of two message sequences,
    returning the fraction of mismatches.
    """
    #print seq1
    #print seq2

    #so this is Hamming distance?

    errors = 0
    for i in range(0,len(seq1)):
        if seq1[i] != seq2[i]:
            errors = errors + 1
    #print errors
    #print (float(errors)/float(len(seq1)))
    return (float(errors)/float(len(seq1)))

def compute_ber(sigma,vth=0.5,v0=0.0,v1=1.0,p0=0.5,p1=0.5):
    """
    Return an estimate of the bit error rate given the values
    for the threshold voltage and the two received voltages
    for 0 and 1.  Use PS4_tests.unit_normal_cdf if you need
    values of &Phi;(x).
    """
    return p0*PS4_tests.unit_normal_cdf((-vth-v0)/sigma)+p1*PS4_tests.unit_normal_cdf((vth-v1)/sigma)


if __name__ == '__main__':
    # make sure functions pass some simple tests
    PS4_tests.test_bit_error_rate(bit_error_rate)
    PS4_tests.test_compute_ber(compute_ber)

    # construct a test message
    message = [random.randint(0,1) for i in xrange(1000000)]

    # try out different noise levels
    ber_values = []
    snr_values = []
    for sigma in (0.5,0.25,0.18,.05):
        noisy_data = PS4_tests.transmit(message,
                                        samples_per_bit=4,
                                        nsigma=sigma)
        received_message = receive(noisy_data)
        ber = bit_error_rate(message,received_message)
        ber_values.append(ber)

        # use 0.25 as power of signal (see lec. slides)
        snr = 10*math.log(0.25/(sigma**2),10)
        snr_values.append(snr)

        print "For sigma = %g" % sigma,
        print "(SNR = %g db):" % snr,
        print "bit error rate = %g," % ber,
        print "computed BER = %g" % compute_ber(sigma)

    """
    # plot BER vs SNR
    p.figure()
    ax = subplot(111)
    p.plot(snr,ber,'b-',lw=2)
    p.title('BER vs SNR')
    ax.set_yscale('log')
    ax.set_ylabel('BER')
    ax.set_xlabel('SNR (db)')
    p.show()
    """
