import numpy,os,wave
import matplotlib.pyplot as p

# leverage FFT to compute fourier series coefficients quickly
def fast_frequency_domain(tseq):
    return list(numpy.fft.fftshift(numpy.fft.fft(numpy.array(tseq)) / len(tseq)))

# leverage IFFT to compute time domain from fourier series coefficients quickly
def fast_time_domain(fseq):
    return list(numpy.real(numpy.fft.ifft(numpy.fft.fftshift(numpy.array(fseq))*len(fseq))))

# simplistic low-pass filter
def band_limit(tseq,k):
    fseq = numpy.array(fast_frequency_domain(tseq))
    N = len(fseq)
    fseq[0:(N/2)-k] = 0  # zero out a_k for k < cuttoff
    fseq[(N/2)+k:] = 0  # zero out a_k for k > cuttoff
    return fast_time_domain(fseq)

# read in .wav file data, convert to sequence of floats
# scaled between -1 and 1
def read_wav_file(filename):
    assert os.path.exists(filename),\
           "file %s doesn't exist" % filename
    wav = wave.open(filename,'rb')
    total_frames = wav.getnframes()
    nchan = wav.getnchannels()
    assert total_frames > 0,\
           "%s doesn't have any audio data!" % self.filename
    sample_rate = wav.getframerate()
    sample_width = wav.getsampwidth()

    # see http://ccrma.stanford.edu/courses/422/projects/WaveFormat/
    if sample_width == 1:
        # data is unsigned bytes, 0 to 255
        dtype = numpy.uint8
        scale = 1/127.0
        offset = -1.0
    elif sample_width == 2:
        # data is signed 2's complement 16-bit samples (little-endian byte order)
        dtype = numpy.int16
        scale = 1/ 32767.0
        offset = 0.0
    elif sample_width == 4:
        # data is signed 2's complement 16-bit samples (little-endian byte order)
        dtype = numpy.int32
        scale = 1/2147483647.0
        offset = 0.0
    else:
        assert False,"%s: unrecognized sample width %d" % (filename,sample_width)

    # read in data
    frames = wav.readframes(total_frames)
    wav.close()

    # only use 1st channel
    signal = numpy.frombuffer(frames,dtype=dtype)
    signal = signal[::nchan]

    # scale data appropriately
    signal = signal*scale + offset

    return list(signal)

# plot spectrum magnitude
def plot_spectrum_magnitude(tseq):
    N = len(tseq)
    spectrum = fast_frequency_domain(tseq)
    x = range(-(N/2),-(N/2)+N)
    p.plot(x,abs(numpy.array(spectrum)))
    p.xlim(x[0],x[-1])
